import shlex
import subprocess
from typing import List, Sequence, Optional, Tuple

# Global debug flag
_DEBUG_MODE = False


def set_debug_mode(enabled: bool) -> None:
    """Set debug mode to enable/disable command logging."""
    global _DEBUG_MODE
    _DEBUG_MODE = enabled


def get_debug_mode() -> bool:
    """Get current debug mode state."""
    return _DEBUG_MODE


#
# Construct the smartctl/btrfs/...  commands safely.
#


def build_smartctl_scan(device: str) -> List[str]:
    """
    Get disks available:
    """
    return shlex.split(f"smartctl --scan {shlex.quote(device)}")


def build_sudo_smartctl_a(device: str) -> List[str]:
    """
    Legacy way to retrieve info
    """
    return shlex.split(f"sudo smartctl -a {shlex.quote(device)}")


def build_sudo_smartctl_l_error(device: str) -> List[str]:
    """
    retrieve errors
    """
    return shlex.split(f"sudo smartctl -l error {shlex.quote(device)}")


def build_sudo_smartctl_x(device: str) -> List[str]:
    """
    SMART overall-health self-assessment test result:
    test in progress
    completed test in power_on_hours
    """
    return shlex.split(f"sudo smartctl -x {shlex.quote(device)}")


def build_sudo_smartctl_t_short(device: str) -> List[str]:
    """
    run short test
    """
    return shlex.split(f"sudo smartctl -t short {shlex.quote(device)}")


def build_sudo_smartctl_t_long(device: str) -> List[str]:
    """
    run long test
    """
    return shlex.split(f"sudo smartctl -t long {shlex.quote(device)}")


def build_sudo_smartctl_l_selftest(device: str) -> List[str]:
    """
    show self-test log (subset of -x )
    """
    return shlex.split(f"sudo smartctl -l selftest {shlex.quote(device)}")


# NVMe CLI command builders (for NVMe disks that don't support smartctl)


def build_nvme_smart_log(device: str) -> List[str]:
    """
    Get SMART log via nvme-cli
    """
    return shlex.split(f"sudo nvme smart-log {shlex.quote(device)}")


def build_nvme_self_test_log(device: str) -> List[str]:
    """
    Get self-test log via nvme-cli
    """
    return shlex.split(f"sudo nvme self-test-log {shlex.quote(device)}")


def build_nvme_device_self_test_status(device: str) -> List[str]:
    """
    Get current device self-test operation status via nvme-cli.
    Uses -s 0h to query the current test status.
    """
    return shlex.split(f"sudo nvme device-self-test {shlex.quote(device)} -s 0h")


def build_nvme_device_self_test_short(device: str) -> List[str]:
    """
    Start short device self-test via nvme-cli.
    Uses -s 1h to start short test.
    """
    return shlex.split(f"sudo nvme device-self-test {shlex.quote(device)} -s 1h")


def build_nvme_device_self_test_long(device: str) -> List[str]:
    """
    Start extended/long device self-test via nvme-cli.
    Uses -s 2h to start extended test.
    """
    return shlex.split(f"sudo nvme device-self-test {shlex.quote(device)} -s 2h")


def build_nvme_start_self_test(device: str, test_type: str) -> List[str]:
    """
    Start self-test via nvme-cli using device-self-test command.
    test_type: "1" for short, "2" for extended/long

    DEPRECATED: Use build_nvme_device_self_test_short() or
    build_nvme_device_self_test_long() instead.
    """
    if test_type == "1":
        return build_nvme_device_self_test_short(device)
    elif test_type == "2":
        return build_nvme_device_self_test_long(device)
    else:
        # Fallback to old command format
        return shlex.split(
            f"sudo nvme self-test {shlex.quote(device)} --test-type={test_type}"
        )


def build_sudo_btrfs_filesystem_show() -> List[str]:
    """
    list all partitions with btrfs; last info in every paragraph
    is: "path /dev/nvme0n1p1"
    """
    return shlex.split(f"sudo btrfs filesystem show")


def build_sudo_btrfs_scrub_status(mountpoint: str) -> List[str]:
    """
    Show if scrub is running. Show errors.
    """
    return shlex.split(f"sudo btrfs scrub status {shlex.quote(mountpoint)}")


def build_sudo_btrfs_scrub_start(mountpoint: str) -> List[str]:
    """
    Start scrub if it is not already running.
    """
    return shlex.split(f"sudo btrfs scrub start {shlex.quote(mountpoint)}")


def build_sudo_btrfs_device_stats(device: str) -> List[str]:
    """
    Show if scrub is running. Show errors.
    """
    return shlex.split(f"sudo btrfs device stats {shlex.quote(device)}")


def build_snapper_list_configs() -> List[str]:
    """
    Snapper operated btrfs mountpoints:
    Drop 1st 3 lines, the rest is rows with:  configname | mountpoint
    """
    return shlex.split(f"snapper list-configs")


def build_sudo_snapper_c_list(configname: str) -> List[str]:
    """
    list all snapshots
    """
    return shlex.split(f"sudo snapper -c {shlex.quote(configname)} list")


def run_command(command: Sequence[str], timeout: int = 10) -> Tuple[int, str]:
    """Execute a command and return (returncode, output).

    Args:
        command: List of command arguments
        timeout: Maximum time to wait for command completion (seconds)

    Returns:
        Tuple of (returncode, combined_stdout_stderr)
    """
    # Debug: print command being executed
    import sys

    cmd_str = " ".join(command)
    if _DEBUG_MODE:
        print(f"\n[DEBUG] >>> Executing: {cmd_str}", file=sys.stderr, flush=True)

    # Log to file for troubleshooting even without --debug
    try:
        with open("/tmp/diskmanager_debug.log", "a") as f:
            f.write(f"{cmd_str}\n")
    except:
        pass

    try:
        result = subprocess.run(
            list(command),
            capture_output=True,
            text=True,
            timeout=timeout,
            check=False,
        )
        output = result.stdout + result.stderr

        # Check for sudo password prompt or authentication failure - only print in debug mode
        if _DEBUG_MODE and result.returncode != 0:
            error_msg = ""
            if "password" in output.lower() or "sudo" in output.lower():
                error_msg = f"\n[ERROR] SUDO PASSWORD REQUIRED: {cmd_str}"
                error_msg += "\n[ERROR] This command requires sudo authentication"
            else:
                error_msg = (
                    f"\n[ERROR] Command failed (exit {result.returncode}): {cmd_str}"
                )

            # Print error in red if terminal supports it
            if sys.stderr.isatty():
                print(f"\033[91m{error_msg}\033[0m", file=sys.stderr, flush=True)
            else:
                print(error_msg, file=sys.stderr, flush=True)

        return result.returncode, output
    except subprocess.TimeoutExpired:
        if _DEBUG_MODE:
            error_msg = (
                f"\n[ERROR] Command timed out after {timeout} seconds: {cmd_str}"
            )
            if sys.stderr.isatty():
                print(f"\033[91m{error_msg}\033[0m", file=sys.stderr, flush=True)
            else:
                print(error_msg, file=sys.stderr, flush=True)
        return 1, f"Command timed out after {timeout} seconds"
    except Exception as e:
        if _DEBUG_MODE:
            error_msg = f"\n[ERROR] Command failed with exception: {cmd_str}\n  {e}"
            if sys.stderr.isatty():
                print(f"\033[91m{error_msg}\033[0m", file=sys.stderr, flush=True)
            else:
                print(error_msg, file=sys.stderr, flush=True)
        return 1, str(e)




from astropy.coordinates import SkyCoord
from astropy import units as u
from astroquery.vizier import Vizier
import matplotlib.pyplot as plt
import numpy as np

from matplotlib.colors import LinearSegmentedColormap

# Target center coordinates
center = SkyCoord('05h36m13s', '-01d12m07s', frame='icrs')

# Vizier query setup: Hipparcos catalog (I/239/hip_main)
Vizier.ROW_LIMIT = 10000
catalog = "I/239/hip_main"

# Search radius
radius = 25 * u.deg

# Query stars around center within radius
result = Vizier.query_region(center, radius=radius, catalog=catalog)
maxmag=5
if result:
    print(result)
    stars = result[0]
    stars = stars[stars['Vmag'] <= maxmag]
    print(stars)
    # Filter stars with magnitude Hp <= 4
    print(stars.colnames)
    #bright_stars = stars[stars['Vmag'] <= 5]
    #mag = bright_stars['Vmag'] #####bright_stars = stars[stars['Hp'] <= 4]


    # Assuming bright_stars with 'B-V' column exists
    bv = stars['B-V']
    norm = plt.Normalize(vmin=np.min(bv), vmax=np.max(bv))
    #colors = plt.cm.rainbow(norm(bv))

    # Create a custom colormap from red to blue (excluding violet)
    #colors_list = [(0, 0, 1), (1, 0, 0)]  # red to blue
    #custom_cmap = LinearSegmentedColormap.from_list('red_blue', colors_list)
    #colors = custom_cmap(norm(bv))


    # Create a custom colormap from red to white to blue
    #colors_list = [(0, 1, 1), (1,1,1), (1, 1, 1), (1,1,1),(1,1,1),  (1,1,1),(1,1,1), (1, 0, 0)]  # red -> white -> blue
    #custom_cmap = LinearSegmentedColormap.from_list('red_white_blue', colors_list)
    # Custom colormap with slight blue at low end, white in middle, red at high end
    colors_list = [(0.5, 0.7, 1), (1, 1, 1), (1, 0, 0)]  # light blue -> white -> red
    custom_cmap = LinearSegmentedColormap.from_list('lightblue_white_red', colors_list)
    colors = custom_cmap(norm(bv))


    sizes = np.log10((maxmag - stars['Vmag']) * 5 + 1) * 20  # +1 to avoid log10(0), scaled for visibility
    sizes = 20 * np.exp(-0.5 * stars['Vmag'])  # smaller sizes for higher magnitudes (fainter stars)
    sizes = (maxmag - stars['Vmag']) ** 2  # Larger size for brighter stars
    # Plot
    plt.figure(figsize=(6,6))
    plt.scatter(stars['RAICRS'], stars['DEICRS'], s=sizes , c=colors)
    #plt.scatter(ra, dec, s=10, c='white')
    plt.gca().set_facecolor('black')
    plt.xlabel('RA (deg)')
    plt.ylabel('DEC (deg)')
    plt.title('Stars up to 4th magnitude near RA 5h36m13s Dec -1°12′7″')
    plt.gca().invert_xaxis()  # RA increases to the left in sky maps
    plt.gca().set_aspect('equal', adjustable='datalim')
    plt.show()
else:
    print("No stars found in the region.")



#!/root/.local/bin/uv run
#
# session optional pam_exec.so seteuid /usr/local/bin/login-notify.py
#
# chmod +x login-notify.py
# EDIT: /etc/pam.d/sshd  AND APPEND:
# session optional pam_exec.so seteuid /path/to/login-notify.py
#
# install notifator???
#
# /// script
# requires-python = ">=3.10"
# dependencies = ["click", "requests"]
# ///

DEBUG_LOG = "/tmp/login-notify-debug.log"


def _debug(msg: str):
    try:
        import datetime

        with open(DEBUG_LOG, "a") as f:
            f.write(f"{datetime.datetime.now()}: {msg}\n")
    except Exception:
        pass


_debug("imports starting")

import os
import socket
import subprocess as sp
import datetime as dt
import configparser
import shutil
from pathlib import Path

import click
import requests

_debug("imports done")

DEFAULT_CONFIG_PATH = Path("~/.config/influxdb/totalconfig.conf").expanduser()
DEFAULT_NTFY_SERVER = "https://ntfy.sh"


def _send_ntfy(
    topic: str,
    message: str,
    title: str | None = None,
    priority: int = 4,
    server: str | None = None,
    username: str | None = None,
    password: str | None = None,
) -> bool:
    server = server or DEFAULT_NTFY_SERVER
    if topic.startswith("http"):
        url = topic
    else:
        url = f"{server.rstrip('/')}/{topic}"
    headers = {"Priority": str(priority)}
    if title:
        headers["Title"] = title
    auth = (username, password) if username and password else None
    response = requests.post(
        url, data=message.encode("utf-8"), headers=headers, auth=auth, timeout=10
    )
    response.raise_for_status()
    return True


def _load_config() -> dict[str, str | None]:
    config: dict[str, str | None] = {
        "server": None,
        "username": None,
        "password": None,
        "topic": None,
    }
    if not DEFAULT_CONFIG_PATH.exists():
        return config
    parser = configparser.ConfigParser()
    parser.read(DEFAULT_CONFIG_PATH)
    if "ntfy login" in parser:
        section = parser["ntfy login"]
        config["server"] = section.get("server")
        config["username"] = section.get("username")
        config["password"] = section.get("password")
        config["topic"] = section.get("topic")
    return config


def _get_log_file() -> str:
    user = os.environ.get("PAM_USER", os.environ.get("USER", "unknown"))
    return f"/tmp/login-notify-{user}.log"


def _log(msg: str):
    try:
        with open(_get_log_file(), "a") as f:
            f.write(f"{msg}\n")
    except Exception as e:
        _debug(f"log failed: {e}")


def _get_mac(ip: str) -> str:
    if ip.startswith("127.") or ip == "::1":
        return "localhost"
    try:
        cmd = f"arping -c 1 -w 1 -r {ip}"
        _log(f"   {cmd} with uid={os.getuid()}")
        res = sp.check_output(cmd.split())
        mac = res.decode("utf8").strip()
        _log(f"   mac: {mac}")
        return mac
    except Exception as e:
        _log(f"   mac lookup failed: {e}")
        return "?"


PAM_FILE = "/etc/pam.d/sshd"
INSTALL_PATH = "/usr/local/bin/login-notify.py"
PAM_LINE = "session optional pam_exec.so seteuid /usr/local/bin/login-notify.py"


@click.group(invoke_without_command=True)
@click.pass_context
def cli(ctx):
    """Login notification via ntfy.

    Script location: /usr/local/bin/login-notify.py
    PAM config: /etc/pam.d/sshd
      session optional pam_exec.so seteuid /usr/local/bin/login-notify.py

    Setup commands:
      sudo ntfy access user topic_x rw
      ntfy pub topic_x "test"
    """
    if ctx.invoked_subcommand is None:
        ctx.invoke(run)


@cli.command()
@click.option(
    "--ntfy-topic", default=None, help="ntfy topic (default: from config or hostname)"
)
@click.option("--dry-run", is_flag=True, help="Print message without sending")
def run(ntfy_topic, dry_run):
    """Send login notification via ntfy."""
    _log("NEWLOG-----------------------------------")
    _log("     start")

    host = socket.gethostname()
    rhost = os.environ.get("PAM_RHOST", "unknown")
    user = os.environ.get("PAM_USER", "unknown")
    pam_type = os.environ.get("PAM_TYPE", "unknown")
    tty = os.environ.get("PAM_TTY", "")

    _log(f"     host={host}, rhost={rhost}, user={user}, type={pam_type}")

    mac = _get_mac(rhost) if rhost != "unknown" else "?"
    time_str = dt.datetime.now().strftime("%a %H:%M:%S")

    parts = [f"type:{pam_type}", f"mac:{mac}"]
    if tty:
        parts.append(f"tty:{tty}")
    parts.append(time_str)
    message = f"from {rhost}\n" + " | ".join(parts)
    title = f"{user}@{host}"

    if dry_run:
        click.echo(f"Dry run - would send:\nTitle: {title}\n{message}")
        return

    config = _load_config()
    topic = ntfy_topic or config.get("topic") or host
    server = config.get("server")
    username = config.get("username")
    password = config.get("password")

    try:
        _send_ntfy(
            topic,
            message,
            title=title,
            server=server,
            username=username,
            password=password,
        )
        click.echo(f"Notification sent to {topic}")
    except Exception as e:
        click.echo(f"Failed to send notification: {e}", err=True)

    _log(f"------------------{dt.datetime.now()}--")
    _log(f"{user}@{host} < {rhost} / type={pam_type} / tty={tty}")


@cli.command()
@click.option("--verbose", "-v", is_flag=True, help="Show detailed debug information")
def install(verbose):
    """Install login-notify.py to system."""
    _debug(f"install() started with verbose={verbose}")

    script_path = Path(__file__).resolve()
    script_stat = script_path.stat()
    script_size = script_stat.st_size
    script_mtime = dt.datetime.fromtimestamp(script_stat.st_mtime).strftime(
        "%Y-%m-%d %H:%M:%S"
    )

    click.echo("Step 1: Checking PAM configuration file...")
    _debug(f"Checking PAM_FILE: {PAM_FILE}")
    if not Path(PAM_FILE).exists():
        click.echo(f"  ERROR: {PAM_FILE} not found", err=True)
        _debug(f"PAM file not found: {PAM_FILE}")
        return
    click.echo(f"  OK: {PAM_FILE} exists")
    _debug(f"PAM file exists: {PAM_FILE}")

    click.echo("\nStep 2: Checking PAM configuration line...")
    pam_content = Path(PAM_FILE).read_text()
    if PAM_LINE in pam_content:
        click.echo(f"  OK: PAM line already present")
        _debug("PAM line found in config")
    else:
        click.echo(f"  WARNING: PAM line not found")
        click.echo(f"  To add manually, append to {PAM_FILE}:")
        click.echo(f"    {PAM_LINE}")
        _debug("PAM line not found in config")

    click.echo("\nStep 3: Checking uv-astral for root...")
    _debug("Checking if uv-astral is installed for root")
    root_uv_path = Path("/root/.local/bin/uv")
    uv_installed = False

    # Check if uv is in root's PATH or at the expected location
    if root_uv_path.exists():
        click.echo(f"  OK: uv found at {root_uv_path}")
        _debug(f"uv found at {root_uv_path}")
        uv_installed = True
    else:
        # Try to find uv in root's PATH
        try:
            result = sp.run(["which", "uv"], capture_output=True, text=True, timeout=5)
            if result.returncode == 0 and result.stdout.strip():
                click.echo(f"  OK: uv found at {result.stdout.strip()}")
                _debug(f"uv found in PATH: {result.stdout.strip()}")
                uv_installed = True
        except Exception:
            pass

    if not uv_installed:
        click.echo("  WARNING: uv-astral is NOT installed for root user!")
        click.echo("  The script will fail when run by PAM (which runs as root).")
        click.echo("")
        click.echo("  To install uv-astral for root, run:")
        click.echo("    sudo -i")
        click.echo("    curl -LsSf https://astral.sh/uv/install.sh | sh")
        click.echo("")

        if click.confirm(
            "  Would you like to install uv-astral for root now?", default=False
        ):
            click.echo("  Installing uv-astral for root...")
            try:
                install_cmd = "curl -LsSf https://astral.sh/uv/install.sh | sh"
                result = sp.run(
                    ["sudo", "-i", "bash", "-c", install_cmd],
                    capture_output=True,
                    text=True,
                    timeout=60,
                )
                if result.returncode == 0:
                    click.echo("  OK: uv-astral installed successfully for root")
                    _debug("uv-astral installed successfully")
                else:
                    click.echo(
                        f"  WARNING: Installation may have failed (exit code {result.returncode})"
                    )
                    if result.stderr:
                        click.echo(f"  Error: {result.stderr}")
                    _debug(f"uv installation failed: {result.stderr}")
            except Exception as e:
                click.echo(f"  ERROR: Failed to install uv-astral: {e}", err=True)
                _debug(f"Exception during uv install: {e}")
        else:
            click.echo(
                "  Skipped. Remember to install uv-astral manually before using login-notify."
            )
            _debug("User skipped uv installation")

    click.echo("\nStep 4: Installing script...")
    click.echo(f"  Current script: {script_size} bytes, {script_mtime}")
    dest = Path(INSTALL_PATH)
    if dest.exists():
        stat = dest.stat()
        mtime = dt.datetime.fromtimestamp(stat.st_mtime).strftime("%Y-%m-%d %H:%M:%S")
        size = stat.st_size
        click.echo(f"  Installed file: {size} bytes, {mtime}")
        if not click.confirm("  Overwrite?", default=False):
            click.echo("  Skipped.")
            _debug("User chose not to overwrite existing file")
            return
    else:
        click.echo(f"  Target: {INSTALL_PATH} (new)")

    try:
        shutil.copy2(script_path, dest)
        dest.chmod(0o755)
        click.echo(f"  OK: Installed to {INSTALL_PATH}")
        _debug(f"Script copied to {INSTALL_PATH}")
    except PermissionError:
        click.echo(f"  ERROR: Permission denied. Run with sudo.", err=True)
        _debug("Permission denied during copy")
        return

    click.echo("\nStep 5: Config file (optional)...")
    _debug(f"Checking config file: {DEFAULT_CONFIG_PATH}")
    if DEFAULT_CONFIG_PATH.exists():
        click.echo(f"  OK: {DEFAULT_CONFIG_PATH} exists")
        _debug(f"Config file found at {DEFAULT_CONFIG_PATH}")

        # Parse and validate config
        click.echo("\nStep 6: Validating config file...")
        config = _load_config()

        if verbose:
            click.echo(f"  DEBUG: Config file contents:")
            try:
                with open(DEFAULT_CONFIG_PATH) as f:
                    for line in f:
                        if "=" in line and not line.strip().startswith("#"):
                            key = line.split("=")[0].strip()
                            click.echo(
                                f"    {key}: {'[SET]' if config.get(key) else '[NOT SET]'}"
                            )
            except Exception as e:
                click.echo(f"    ERROR reading config: {e}", err=True)

        # Check required fields
        checks = {
            "server": config.get("server"),
            "username": config.get("username"),
            "password": config.get("password"),
            "topic": config.get("topic"),
        }

        all_ok = True
        for key, value in checks.items():
            if value:
                click.echo(f"  OK: {key} is set")
                _debug(f"Config {key} is set")
            else:
                click.echo(f"  WARNING: {key} is not set (will use defaults)")
                _debug(f"Config {key} is not set")
                all_ok = False

        if all_ok:
            click.echo("  All configuration values are set!")
        else:
            click.echo("  Note: Some values use defaults. Edit config to customize.")
    else:
        click.echo(f"  WARNING: {DEFAULT_CONFIG_PATH} not found")
        click.echo(f"  Create {DEFAULT_CONFIG_PATH} with:")
        click.echo("    [ntfy login]")
        click.echo("    server = https://ntfy.sh")
        click.echo("    username = your_user")
        click.echo("    password = your_password")
        click.echo("    topic = ssh_logins")
        _debug("Config file not found")

    click.echo("\nInstallation complete.")
    _debug("install() completed successfully")


if __name__ == "__main__":
    cli()



#!/usr/bin/env python3


# -----------  jsut checking number of parameters.
# --- this script is called from py wrapper
# --- and paramters are passed using sys.argv
# --- and compatible with uv uvx
#

import subprocess as sp
import shlex
import os
import sys
import click

from jusfltuls.check_new_version import is_there_new_version
from importlib.metadata import version as pkg_version

def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    print(f"distcheck {pkg_version('jusfltuls')}")
    is_there_new_version(package="jusfltuls", printit=True, printall=True)
    ctx.exit()

@click.command(context_settings=dict(help_option_names=['-h', '--help']))
@click.option('-v', '--version', is_flag=True, callback=print_version, expose_value=False, is_eager=True, help='Show version and exit')
def main():
    is_there_new_version(package="jusfltuls", printit=True, printall=True)
    command=f"distcheck.sh"

    script_dir = os.path.dirname(os.path.abspath(__file__))
    print("D... ...", script_dir)
    full_command = os.path.join(script_dir, command)
    print("D... ...", full_command)
    CMD = shlex.split(full_command)
    # it is interactive... FULL INTERACTIVITY
    result = sp.run( CMD )#, capture_output=True, text=True )
    #if result.returncode != 0:
    #    print("X... error calling", command)
    #print(result.stdout ) ####   REPOERT THE OUTPUT
    return

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""Wrapper module for login-notify install command."""

import subprocess
import sys
from pathlib import Path


def install():
    """Entry point for login-notify-install command."""
    script_path = Path(__file__).parent / "login-notify.py"
    result = subprocess.run([sys.executable, str(script_path), "install"])
    sys.exit(result.returncode)


if __name__ == "__main__":
    install()



import curses
import click
import logging
import random

import jusfltuls.mcinfl_library as maninf
#from  jusfltuls.mcinfl_library import user_glob # set this for all session
import sys
import datetime as dt
import os
import time
from console import fg, bg, fx
import socket
from jusfltuls.check_new_version import is_there_new_version
from importlib.metadata import version as pkg_version

"""
1.    -portable   -skip-errors   sometimes not possibnle

~/.visidatarc
# ------------------------------------------------------------
# RUNWITH  uvx --with=numpy visidata -p export_local_flashcam5000_20250522_0838_01.vdj
options.color_key_col=''
options.color_selected_row='yellow'
options.color_note_type='yellow'
#options.color_graph_hidden='blue'
options.color_column_sep='blue'
# COMMAND FOR '1'
BaseSheet.addCommand('1', 'hello-world1', 'status("Hello world! C-D saves and -p saved.vdj replays; z| sel ...; gt; gd; ")')
import numpy as np
import datetime as dt
BaseSheet.addCommand('2', 'hello-world2', 'status(f"Random sample: {np.random.sample()}  {dt.datetime.now().strftime(\'%t\')}")')
def t2d(time):
    return dt.datetime.fromtimestamp(time/1000000000)


"""



HELP_NEWDB = """
    1.  NEW INFLUXDB
      - TO CREATE A USER AND PRIVILEGE:
    nano /etc/influxdb/influxdb.conf ; auth-enabled = false ;
    influx ; create user abc with password 'def' with all privileges;
    nano /etc/influxdb/influxdb.conf; auth-enabled = true;
    systemctl restart influxd

    2. REMOTE SERVER CONFIGURATION (~/.config/influxdb/totalconfig.conf)
    [influx public]
    server=inf.example.com
    username=user
    password=mypass
    username_admin=admin
    password_admin=myadminpass

    Then run: uv run mcinfl --host public  (or -H public)
    ---------------------------------------------------------------------------
"""

DISK_MIRROR_PATH = os.path.expanduser('~/INFLUXDB_MIRROR')

# In curses_main add state for disk mode per panel
# Add variables at start of curses_main:
left_disk_mode = False
right_disk_mode = False


def rename_with_extension_preserved(old_name, new_name):
    logging.debug(f" renaming_w_e_p  {old_name} to {new_name} ")

    base, ext = os.path.splitext(old_name.rstrip("/")) # if folder, I strip / to see ext
    mynew_name = new_name
    if not mynew_name.endswith(ext):
        logging.debug(f"  ext was  /{ext}/ BUT it doesnt match  ")
        mynew_name += ext
    logging.debug(f"  ext was  {ext} and new name is   {mynew_name} ")
    return mynew_name



# ================================================================================
#    panels status  +disk
# --------------------------------------------------------------------------------

def get_disk_panel_status():
    if not os.path.exists(DISK_MIRROR_PATH):
        os.makedirs(DISK_MIRROR_PATH)
    try:
        items = os.listdir(DISK_MIRROR_PATH)
    except Exception as e:
        items = []
    # Mark directories with trailing slash for clarity
    items = [f + '/' if os.path.isdir(os.path.join(DISK_MIRROR_PATH, f)) else f for f in items]
    return sorted(items)



# Modify get_left_panel_status and get_right_panel_status to check disk mode:
def get_left_panel_status():
    if left_disk_mode:
        return get_disk_panel_status()
    else:
        return maninf.show_databases()

def get_right_panel_status():
    if right_disk_mode:
        return get_disk_panel_status()
    else:
        return maninf.show_databases()


# def get_left_panel_status():
#     res = maninf.show_databases()
#     logging.debug(f"{res}")
#     return res#["file1.txt", "file2.txt", "dir1/", "file3.log"]

# def get_right_panel_status():
#     res = maninf.show_databases()
#     logging.debug(f"{res}")# return ["doc1.pdf", "image.png", "music.mp3", "video.mp4"]
#     return res
#     #return ["doc1.pdf", "image.png", "music.mp3", "video.mp4"]


# ================================================================================
#  JUST A MESSAGE
# --------------------------------------------------------------------------------

def show_message_box(stdscr, message, wait_key=True):
    if message is None:
        return
    lines = [line.strip() for line in message.strip().split('\n')]
    if wait_key:
        lines.append("press-a-key")
    h, w = stdscr.getmaxyx()
    #
    box_h = 8
    if len(lines) > 6 and len(lines) < h - 3:
        box_h = len(lines) + 2
    elif len(lines) > 6:
        box_h = h - 1
    box_w = max(len(line) for line in lines) + 4
    if box_w > w - 2:
        box_w = w - 2
    #
    win = curses.newwin(box_h, box_w, (h - box_h)//2, (w - box_w)//2)
    win.box()
    for i, line in enumerate(lines[:box_h - 2]):
        win.addstr(1 + i, 2, line[:box_w - 4])
    win.refresh()
    if wait_key:
        win.getch()  # wait for key press
    win.clear()
    stdscr.refresh()


# ================================================================================
#  CONFIRM   MESSAGE
# --------------------------------------------------------------------------------


def confirm_dialog(stdscr, prompt):
    h, w = stdscr.getmaxyx()
    pro = prompt.split("\n")
    lines = len(pro)
    lenpromptmax = 1
    for i in pro:
        if len(i) > lenpromptmax:
            lenpromptmax =len(i)

    win = curses.newwin(5 + lines, lenpromptmax + 12, h//2 - 2, (w - lenpromptmax - 12)//2)
    win.box()
    ii = 0
    for i in pro:
        win.addstr(1 + ii, 3, i)
        ii += 1
    win.addstr(3 + lines, 4, "Yes (y) / No (n)")
    win.refresh()
    while True:
        c = win.getch()
        if c in (ord('y'), ord('Y')):
            return True
        elif c in (ord('n'), ord('N')):
            return False

# ================================================================================
#
# --------------------------------------------------------------------------------

def on_backup(stdscr):
    """
     RUN  influxd backup  -portable  to the hardcoded PATH
    """
    NOW = dt.datetime.now().strftime("%Y%m%d_%H%M")
    #PATH = f"~/INFL_BACKUP_PORTABLE_{NOW}"
    PATH = f"{DISK_MIRROR_PATH}/_full_backup_{NOW}"
    #newname = input_dialog(stdscr, f"Backup Full Influx? -portable -skip-errors")
    confirm = confirm_dialog(stdscr, f"Backup Full Influx? -portable -skip-errors; {PATH}")
    if confirm:
        PATH = os.path.expanduser(PATH)
        if os.path.exists(PATH):
            pass
        else:
            os.mkdir(PATH)
        logging.debug(f"Started  backup to {PATH}")
        maninf.backup_portable(PATH)
        logging.debug(f"Done backup to {PATH}")

# ================================================================================
#
# --------------------------------------------------------------------------------

def on_copy(name, dbase, other_dbase, stdscr, wordcopy="COPY", asking=True,
            silent=True,
            new_measurement=None
            ):
    """
    I try to have this as the only interface to all copy_measurement calls. This means +silent+new_measurement
    """
    logging.debug(f"copying - {name} from {dbase} -> db:{other_dbase};")
    if stdscr:
        confirm = True
        if asking:
            confirm = confirm_dialog(stdscr, f"{wordcopy} measurement '{name}' of {dbase} TO {other_dbase}?")
        if confirm:
            if (name is None) or (dbase is None) or (other_dbase is None):
                pass
            else:
                show_message_box(stdscr, f"... performing {wordcopy} ...", wait_key=False)
                #
                res = maninf.copy_measurement2(name, dbase, other_dbase, silent=silent, new_measurement=new_measurement)
                #res = maninf.copy_measurement(name, dbase, other_dbase, silent=silent, new_measurement=new_measurement)
                ##### res = maninf.copy_measurement(name, dbase, dbase, silent=True, new_measurement_name=new_name) # rename part
                #
                show_message_box(stdscr, f"... {wordcopy} done ...", wait_key=False)
                logging.debug( f"{res}")
            logging.debug(f"{wordcopy}-ed '{name}'")
        else:
            logging.debug(f"{wordcopy}-ing cancelled for '{name}'")
    else:
        # If no stdscr provided, assume yes or handle differently
        # maninf.drop_database(name)
        logging.debug(f"{wordcopy}-ed '{name}' without confirmation")
    pass

# ================================================================================
#
# --------------------------------------------------------------------------------

def on_move():
    logging.debug(f"moving - ;")
    pass

def on_create_database(name):
    logging.debug(f"creating new thing - {name};")
    if name is None or len(name) == 0:
        return
    maninf.create_database(name)
    pass

def on_delete_database(name, stdscr):
    logging.debug(f"deleting database - {name};")
    if stdscr:
        confirm = confirm_dialog(stdscr, f"Delete database '{name}'?")
        if confirm:
            show_message_box(stdscr, "... preforming drop db ...", wait_key=False)
            maninf.drop_database(name)
            logging.debug(f"Deleted '{name}'")
        else:
            logging.debug(f"Deletion cancelled for '{name}'")
    else:
        # If no stdscr provided, assume yes or handle differently
        # maninf.drop_database(name)
        logging.debug(f"Deleted '{name}' without confirmation")


def on_delete_measurement(name, database, stdscr, asking=True):
    """
    I took the same as delete database
    """
    logging.debug(f"deleting measurement - {name} from {database};")
    if stdscr:
        confirm = True
        if asking:
            confirm = confirm_dialog(stdscr, f"Delete measurement '{name}' of {database}?")
        if confirm:
            show_message_box(stdscr, "... preforming delete ...", wait_key=False)
            maninf.delete_measurement(name, database)
            logging.debug(f"Deleted measurement '{name}' from {database}")
        else:
            logging.debug(f"Deletion cancelled for meas. '{name}' of {database}")
    else:
        # If no stdscr provided, assume yes or handle differently
        # maninf.drop_database(name)
        logging.debug(f"Deleted '{name}' without confirmation")



def draw_panel(win, items, selected_idx, active, offset, disk_mode=False):
    """
     updated for offset to enable scrolling; updated to show mode
    """
    h, w = win.getmaxyx()
    win.clear()
    win.box()
    # Draw mode indicator inside the top border line, centered
    mode_text = "[DISK MODE]" if disk_mode else "[INFLUXDB]"
    #mode_text = mode_text.center(w - 2)
    try:
        # Add mode text on top border line at row 0, col 1 (inside box)
        win.addstr(0, w - len(mode_text) * 2, mode_text, curses.A_BOLD | (curses.A_REVERSE if active else curses.A_DIM))
    except curses.error:
        pass
    visible_items = items[offset:offset + h - 2]
    for idx, item in enumerate(visible_items):
        real_idx = idx + offset
        if real_idx == selected_idx:
            mode = curses.A_REVERSE if active else curses.A_BOLD
        else:
            mode = curses.A_NORMAL
        try:
            win.addstr(idx + 1, 1, item[:w-3], mode)
        except curses.error:
            pass
    win.refresh()

def draw_bottom_bar(stdscr):

    curses.start_color()
    curses.init_pair(1, curses.COLOR_BLACK, curses.COLOR_CYAN)  # text black, background cyan
    curses.init_pair(2, curses.COLOR_WHITE, curses.COLOR_RED)   # text white, background red for admin

    h, w = stdscr.getmaxyx()
    #
    #
    bar_text = " 2 Export  (s-)3 View  4 Insert  5 Copy  6 Move/RenameDB  7 Create  8 Delete  9 Grant [b-backup r-refresh d-diskpanel] "

    # Show server name for remote, hostname for local
    if maninf.HOSTNAME == 'local':
        display_name = socket.gethostname()
    else:
        # Get server name from config for remote connections
        u, p, server = maninf.get_user_pass()
        display_name = f"{server}({maninf.HOSTNAME})"

    # Reserve space for display_name plus a space before it
    max_bar_width = w - len(display_name) - 1
    bar_text = bar_text[:max_bar_width]
    bar_text = bar_text.ljust(max_bar_width) + " " + display_name
    #
    #
    stdscr.attron(curses.color_pair(1))
    #stdscr.attron(curses.A_REVERSE)

    # Truncate bar_text if wider than screen width
    text = bar_text[:w]
    try:
        # Draw most of the bar normally
        if maninf.user_glob_admin:
            # In admin mode, highlight the server/hostname part
            main_part = bar_text[:max_bar_width]
            stdscr.addstr(h-1, 0, main_part.ljust(max_bar_width))
            stdscr.attroff(curses.color_pair(1))
            stdscr.attron(curses.color_pair(2))
            stdscr.addstr(h-1, max_bar_width, " " + display_name)
            stdscr.attroff(curses.color_pair(2))
        else:
            stdscr.addstr(h-1, 0, text.ljust(w))
    except curses.error:
        # In case window too small, ignore error
        pass
    #stdscr.attroff(curses.A_REVERSE)
    stdscr.attroff(curses.color_pair(1))
    stdscr.refresh()

def input_dialog(stdscr, prompt):
    curses.echo()
    h, w = stdscr.getmaxyx()
    win = curses.newwin(3, w//2, h//2 - 1, w//4)
    win.box()
    win.addstr(1, 2, prompt)
    win.refresh()
    stdscr.refresh()
    curses.curs_set(1)
    input_win = curses.newwin(1, w//2 - len(prompt) - 4, h//2, w//4 + len(prompt) + 2)
    curses.echo()
    input_win.clear()
    input_win.refresh()
    s = input_win.getstr().decode('utf-8')
    curses.noecho()
    curses.curs_set(0)
    return s

# ================================================================================
#    MAIN *****************************
# --------------------------------------------------------------------------------
#    MAIN *****************************
# --------------------------------------------------------------------------------
#    MAIN *****************************
# --------------------------------------------------------------------------------

def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    print(f"mcinfl {pkg_version('jusfltuls')}")
    is_there_new_version(package="jusfltuls", printit=True, printall=True)
    ctx.exit()

@click.command(context_settings=dict(help_option_names=['-h', '--help']))
@click.option('-v', '--version', is_flag=True, callback=print_version, expose_value=False, is_eager=True, help='Show version and exit')
@click.option('--logfile', default='/tmp/mcinflux_debug.log', help='Log file path')
@click.option('--admin', "-a", is_flag=True, help='Act as admin. Else user privileges are used')
@click.option('--test_only', "-t", is_flag=True, help='Test, do not run the commander itself (normally connects after)')
@click.option('--skip_test', "-s", is_flag=True, help='Skip all testing and try to connect')
@click.option('--host', "-H", default='local', show_default=True,  help='Target host: possibilities [local(default) | localnet | public ]')
def main(logfile, admin, test_only, skip_test, host):
    """
    InfluxDB Commander - Interactive dual-panel interface for InfluxDB management.

    To test the database setup, use -t -a options together.
    """

    global user_glob_admin
    global left_disk_mode, right_disk_mode
    # trick to have systemwide LOG
    maninf.LOGFILENAME = logfile
    # trick to have systemwide HOST .....
    maninf.HOSTNAME = host

    # check PyPI for version
    is_there_new_version(package="jusfltuls", printit=True, printall=True)

    #print("Hey, machine is expected in the format like '127.0.0.1' or 'www.example.com' (where --ssl is selected) ")

    logging.basicConfig(filename=logfile, level=logging.DEBUG,
                        format='%(asctime)s - %(levelname)s - %(message)s')
    logging.debug('Debug log started')


    #res = maninf.verify_files(justprint=False) # verify files needed (one now) without printing if OK
    #if not res: sys.exit(1)

    # NOTMALLY user_glob_admin == True    BUT FORCEFULLY
    maninf.user_glob_admin = admin # TRUE OR FALSE   set this for all session

    if admin: # SWITCH IS ACTIVATED
        print(f"i... switch 'admin' is {admin}  ( switch login TO  admin) ")
        print(f"i... global var 'userg' is {maninf.user_glob_admin}")

    #
    # check for total config
    ok = maninf.verify_files()
    if not ok:
        print("X...  files missing")
        # CREATE FILES AND MKDIR !!!!
        sys.exit(1)


    # ---this is the way user is taken....
    u, p, serv =maninf.get_user_pass()
    if u is None or p is None:
        print("X... user or pass problem", u)
        sys.exit(1)
    #else:        print(f"/{u}/, /{p}/")

    m =serv  #maninf.get_local_machine()
    if m is None:
        print("X... machine name problem")
        sys.exit(1)

    print("i... pentesting... ", end="")
    res = maninf.show_databases(dontusepass=True, silent=True)
    if len(res) > 0:
        print(f"{fg.red} FAILED {fg.default} ")
        print(f"X... {fg.red}INFLUX SEEMS TO HAVE  NO USER AND PASSWORD ... {fg.default}")
        print(HELP_NEWDB)
        if not skip_test:
            sys.exit(1)
        else:
            print(".... skipping exit ", end="")
    print(f"{fg.green} OK {fg.default} ")  # pentesting OK

    print(f"i... user: {fg.green}{u}{fg.default} on machine: {fg.yellow}{m}{fg.default} , I am going IN")  # ******************<<<<<



    # ONLY THING USER CAN **************
    res = maninf.show_databases()
    # I THINK I AM IN ************************** ****************************************************************************

    if len(res) == 0:
        print(f"X... {fg.red} NO DATABASES seen !! {fg.default}   .........   THOROUGH TESTING STARTS .....")


        # ADMIN ONLY ********************************************
        print("i... grant testing (requires admin)... ", end="")
        res2 = maninf.show_grants(silent=False)
        if res2 is None:  # --------------------------------------------- ABNORMAL TESTING -----------------------------------------
            print(f"{fg.red} FAILED {fg.default} ")
            u, p, serv =maninf.get_user_pass()
            print(f"D...                user {fg.orange} {u} {fg.default} seems not to be admin ")
            print(f"D... no privileges for    {fg.orange}{u}{fg.default}    user")
            if u == "admin":
                print(f"X... since this is admin user, there is some {fg.red}problem{fg.default} here")

            # ---- since previous is non admin, I change USER  -----------------------------
            print(f"x... ... {fg.magenta}I am trying to change-switch USER,  changing user {fg.default} ")
            print(f"x... ... {fg.magenta}------------------------------------------------- {fg.default} ")
            time.sleep(1)
            #
            maninf.user_glob_admin = not admin   # set this for all session
            #print(f"i... variable 'userg' is INVERTED TO  {maninf.user_glob_admin}")
            #
            u, p, serv =maninf.get_user_pass()
            print(f"i... user now is   {fg.orange}{u}{fg.default}  ")
            if u is None or p is None:
                print(f"X... {fg.red}user or passwd problem for the switched user ...{fg.defauilt}")
                maninf.is_infl_active()
                sys.exit(1)
            else:
                print(f"D...  user/pass seems    here seems {fg.green}ok{fg.default}")
            #else:        print(f"/{u}/, /{p}/")

            m =serv  #maninf.get_local_machine()

            if m is None:
                print(f"X... {fg.red}machine name problem{fg.default}")
                sys.exit(1)
            else:
                print(f" D...  machine   seems   here seems {fg.green}ok{fg.default}")

            res2 = maninf.show_grants(silent=True)
            if res2 is None:
                print(f"X... also the   {u}  user   is not admin user ")
            else:# ADMIN HERE
                print(f"i... user {fg.orange}{u}{fg.default} must be (seems to be) {fg.cyan}ADMIN{fg.default} " )
                print(f"i... user {fg.orange}{u}{fg.default} has privileges to databases: {list(res2.keys())}" )
                if "ALL PRIVILEGES" in list(res2.values()):
                    print(f"I... USER {fg.orange}{u}{fg.default} has {fg.cyan}SOME MAXIMUM {fg.default} privileges!\n     ", res2)
                    #if u != "admin": # normal
                    #    print(f"X... {fg.red} TAKE CARE ABOUT NON ADMIN USER WITH MAX PRIVILEGES! {fg.default}")
                print("i... users show   (requires admin) ....")
                res2 = maninf.show_users(silent=True)
                print("D... all users list: ", res2)
                if not "admin" in list(res2.keys()):
                    print(f"X... {fg.red} 'admin'  USER NOT  PRESENT {fg.default}")
                for i in res2.keys():
                    if i != "admin" and res2[i] == "ADMIN":
                        print(f"X... {fg.red} '{i}' user is ADMIN {fg.default}")
            #---------------------------------- res2 ----
            print(f"i... end of the abnormal testing")
            if not skip_test:
                sys.exit(1) # --------------------------------------------- ABNORMAL TESTING -----------------------------------------
            else:
                print(".... skipping exit ", end="")


        print(f"{fg.green} OK {fg.default} ")  # pentesting OK
        # GRANTS OK -------------------------


        # ADMIN ONLY ***************************************
        print("i... users show... ", end="")
        res2 = maninf.show_users(silent=True)
        print(f"{len(res2)} users found ... {list(res2.keys())}")
        if not "admin" in list(res2.keys()):
            print(f"X... {fg.red} 'admin'  USER NOT  PRESENT {fg.default}")
        for i in res2.keys():
            if i != "admin" and res2[i] == "ADMIN":
                print(f"X... {fg.red} '{i}' user is ADMIN {fg.default}")


        # -------------------------------------------------------- FORCE QUIT WHEN USER SWITCH
        if not skip_test:
            sys.exit(1) # --------------------------------------------- ABNORMAL TESTING -----------------------------------------
        else:
            print(".... skipping exit ", end="")
        #sys.exit(0)

    print("i... initial sequence finished.")
    if test_only:
        print("""
        TRICK TO MAKE BACKUP
        sudo influxd restore -portable -host 127.0.0.1:8088 /root/influx-backup` WORKED when db 1.8.4 is live!
        TRICK TO CONNECT
        influx -host 127.0.0.1 -p 8086  # and use auth....
        """)
        sys.exit(0)

    def curses_main(stdscr):

        global left_disk_mode, right_disk_mode

        #print("Ciao")
        #logging.debug("Ciao")
        left_db_selected = None
        right_db_selected = None

        # previous idx==remember where to return
        left_idx_prev = 0
        right_idx_prev = 0

        left_offset, right_offset = 0, 0

        curses.curs_set(0)
        stdscr.clear()
        stdscr.refresh()
        left_items = get_left_panel_status()
        right_items = get_right_panel_status()
        left_idx, right_idx = 0, 0
        active_panel = 'left'

        h, w = stdscr.getmaxyx()
        mid = w // 2

        left_win = curses.newwin(h-1, mid, 0, 0)
        right_win = curses.newwin(h-1, w - mid, 0, mid)


        # Modify refresh_panels to use updated get_left_panel_status and get_right_panel_status
        def refresh_panels():
            nonlocal left_items, right_items, left_idx, right_idx
            global left_disk_mode, right_disk_mode

            if left_disk_mode:
                left_items = get_disk_panel_status()
            else:
                if left_db_selected is None:
                    left_items = maninf.show_databases()
                else:
                    left_items = ['..'] + maninf.show_measurements(left_db_selected)
            if right_disk_mode:
                right_items = get_disk_panel_status()
            else:
                if right_db_selected is None:
                    right_items = maninf.show_databases()
                else:
                    right_items = ['..'] + maninf.show_measurements(right_db_selected)
            left_idx = min(left_idx, len(left_items) - 1) if left_items else 0
            right_idx = min(right_idx, len(right_items) - 1) if right_items else 0

        # def refresh_panels():
        #     nonlocal left_items, right_items, left_idx, right_idx
        #     if left_db_selected is None:
        #         left_items = get_left_panel_status()
        #     else:
        #         left_items = ['..'] + maninf.show_measurements(left_db_selected)
        #     if right_db_selected is None:
        #         right_items = get_right_panel_status()
        #     else:
        #         right_items = ['..'] + maninf.show_measurements(right_db_selected)
        #     left_idx = min(left_idx, len(left_items) - 1) if left_items else 0
        #     right_idx = min(right_idx, len(right_items) - 1) if right_items else 0

        refresh_panels()

        # older
        #draw_panel(left_win, left_items, left_idx, active_panel == 'left')
        #draw_panel(right_win, right_items, right_idx, active_panel == 'right')
        # newer with scrolling
        draw_panel(left_win, left_items, left_idx, active_panel == 'left', left_offset, disk_mode=left_disk_mode)
        draw_panel(right_win, right_items, right_idx, active_panel == 'right', right_offset, disk_mode=right_disk_mode)
        #
        draw_bottom_bar(stdscr)

        while True:
            key = stdscr.getch()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            if key == 9:  # TAB
                logging.debug(f" key TAB pressed   ")
                active_panel = 'right' if active_panel == 'left' else 'left'
                if active_panel == 'left':
                    left_idx = max(0, min(left_idx, len(left_items) - 1))
                else:
                    right_idx = max(0, min(right_idx, len(right_items) - 1))

            # ---------------------------------------- original movement in panels

            # elif key == curses.KEY_UP:
            #     if active_panel == 'left':
            #         left_idx = max(0, left_idx - 1)
            #     else:
            #         right_idx = max(0, right_idx - 1)

            # elif key == curses.KEY_DOWN:
            #     if active_panel == 'left':
            #         left_idx = min(len(left_items) - 1, left_idx + 1)
            #     else:
            #         right_idx = min(len(right_items) - 1, right_idx + 1)


            # new panel movement with scrolling
            elif (key == curses.KEY_UP) and active_panel == 'left':
                logging.debug(f" key UP l   pressed   ")
                if left_idx > 0:
                    left_idx -= 1
                    if left_idx < left_offset:
                        left_offset -= 1
            elif (key == curses.KEY_DOWN) and active_panel == 'left':
                logging.debug(f" key DOWN l  pressed   ")
                if left_idx < len(left_items) - 1:
                    left_idx += 1
                    h, _ = left_win.getmaxyx()
                    if left_idx >= left_offset + h - 2:
                        left_offset += 1
            elif (key == curses.KEY_UP) and active_panel == 'right':
                logging.debug(f" key UP r  pressed   ")
                if right_idx > 0:
                    right_idx -= 1
                    if right_idx < right_offset:
                        right_offset -= 1
            elif (key == curses.KEY_DOWN) and active_panel == 'right':
                logging.debug(f" key DOWN r  pressed   ")
                if right_idx < len(right_items) - 1:
                    right_idx += 1
                    h, _ = right_win.getmaxyx()
                    if right_idx >= right_offset + h - 2:
                        right_offset += 1

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            # ------------------------------------------------------------
            elif key == ord('2'):# --------------------------------- export measurement to csv file
                # EXPORT TO WHERE YOU STAND NOW
                logging.debug(f" key {chr(key)} pressed   ")
                if (active_panel == 'left' and len(left_items) == 0 ) or (active_panel == 'right' and len(right_items) == 0 ):
                    pass
                elif (active_panel == 'left' and left_disk_mode) or (active_panel == 'right' and right_disk_mode):
                    show_message_box(stdscr, "FEATURE NOT IMPLEMENTED")
                    pass
                else:
                    name = None
                    dbase = None
                    if active_panel == 'left':
                        name = left_items[left_idx]
                        dbase = left_db_selected
                        other_dbase = right_db_selected
                    else:
                        name = right_items[right_idx]
                        dbase = right_db_selected
                        other_dbase = left_db_selected

                    res = maninf.export_measurement(name, dbase, silent=True) # This is measurement to csv ORIGINAL WAY !
                    logging.debug(f"{res}")
                    #show_message_box(stdscr, f"{res}")
                refresh_panels()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('3'):# --------------------------------- View measurement
                logging.debug(f" key {chr(key)} pressed   ")
                if (active_panel == 'left' and len(left_items) == 0 ) or (active_panel == 'right' and len(right_items) == 0 ):
                    pass
                elif (active_panel == 'left' and left_disk_mode):  #  LEFT DISK  VIEW

                    #show_message_box(stdscr, "FEATURE NOT IMPLEMENTED")
                    selected_item = left_items[left_idx]
                    path = os.path.join(DISK_MIRROR_PATH, selected_item.rstrip('/'))
                    if selected_item == '..':
                        # Go up one directory if inside subdir, else stay
                        parent = os.path.dirname(DISK_MIRROR_PATH)
                        # For simplicity, stay in root mirror folder (no going above)
                        pass
                    elif os.path.isdir(path):
                        # Optional: implement subdir navigation if desired
                        text = ""
                        if selected_item.find(".exported") > 0:
                            nm1 = selected_item.split(".exported")[0]
                            filepath = os.path.join(DISK_MIRROR_PATH, selected_item)
                            oldname = maninf.restore_portable_get_oldname(filepath, database=nm1)
                            text = f"... exported database: \n {nm1} \n ... prev name: \n {oldname}"
                            show_message_box(stdscr, f"Directory: \n {selected_item} \n {text}")
                    elif os.path.isfile(path):
                        try:
                            with open(path, 'r') as f:
                                content = f.read(1024)
                            show_message_box(stdscr, f"File: {selected_item}\n\n{content}")
                        except Exception as e:
                            show_message_box(stdscr, f"Error reading file: {e}")
                    pass

                elif (active_panel == 'right' and right_disk_mode): # RIGHT DISK VIEW
                    #show_message_box(stdscr, "FEATURE NOT IMPLEMENTED")
                    selected_item = right_items[right_idx]
                    path = os.path.join(DISK_MIRROR_PATH, selected_item.rstrip('/'))
                    if selected_item == '..':
                        # Go up one directory if inside subdir, else stay
                        parent = os.path.dirname(DISK_MIRROR_PATH)
                        # For simplicity, stay in root mirror folder (no going above)
                        pass
                    elif os.path.isdir(path):
                        # Optional: implement subdir navigation if desired
                        text = ""
                        if selected_item.find(".exported") > 0:
                            nm1 = selected_item.split(".exported")[0]
                            filepath = os.path.join(DISK_MIRROR_PATH, selected_item)
                            oldname = maninf.restore_portable_get_oldname(filepath, database=nm1)
                            text = f"... exported database: \n {nm1} \n ... prev name: \n {oldname}"
                            show_message_box(stdscr, f"Directory: \n {selected_item} \n {text}")
                    elif os.path.isfile(path):
                        try:
                            with open(path, 'r') as f:
                                content = f.read(1024)
                            show_message_box(stdscr, f"File: {selected_item}\n\n{content}")
                        except Exception as e:
                            show_message_box(stdscr, f"Error reading file: {e}")
                    pass
                else:
                    name = None
                    dbase = None
                    if active_panel == 'left':
                        name = left_items[left_idx]
                        dbase = left_db_selected
                        other_dbase = right_db_selected
                    else:
                        name = right_items[right_idx]
                        dbase = right_db_selected
                        other_dbase = left_db_selected

                    res = maninf.show_measurement_newest_sample(name, dbase, silent=True)
                    logging.debug(f"{res}")
                    show_message_box(stdscr, f"{res}")
                refresh_panels()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('#'):#   shift-3------------------- VIEW measurement continuously
                logging.debug(f" key {chr(key)} pressed   ")
                if (active_panel == 'left' and len(left_items) == 0 ) or (active_panel == 'right' and len(right_items) == 0 ):
                    pass
                elif (active_panel == 'left' and left_disk_mode) or (active_panel == 'right' and right_disk_mode):
                    show_message_box(stdscr, "FEATURE NOT IMPLEMENTED")
                    pass
                else:
                    name = None
                    dbase = None
                    if active_panel == 'left':
                        name = left_items[left_idx]
                        dbase = left_db_selected
                        other_dbase = right_db_selected
                    else:
                        name = right_items[right_idx]
                        dbase = right_db_selected
                        other_dbase = left_db_selected

                    logging.debug(f"repeating view ....")
                    stdscr.nodelay(True)  # Non-blocking input
                    time_restrict = dt.datetime.now().isoformat() + "Z"  # ISO 8601 format with Zulu time
                    #
                    local = dt.datetime.now().astimezone()  # local time with timezone info
                    utc_dt = local.astimezone(dt.timezone.utc)
                    time_restrict = utc_dt.isoformat().replace("+00:00", "Z")
                    #time_restrict = dt.datetime.now().strftime("%Y-%m-%d %H:%M:%S")#isoformat() + "Z"  # ISO 8601 format with Zulu time
                    time1 = dt.datetime.now().strftime("%H:%M:%S")
                    logging.debug(f"repeating view ....{time_restrict}")

                    while True:
                        time2 = dt.datetime.now().strftime("%H:%M:%S") # interesting. when time1 inside strftime, overrided
                        res = maninf.show_measurement_newest_sample(name, dbase, silent=True, prepend=f"measurement from {time1}  ==>>>  {time2}", time_restrict=time_restrict)
                        show_message_box(stdscr, f"{res}", wait_key=False)
                        time.sleep(2)
                        newkey = -1
                        ####newkey = stdscr.getch()
                        try:
                            newkey = stdscr.getch()
                        except curses.error:
                            newkey = -1
                        if newkey != -1:  # Any key pressed
                            break
                        refresh_panels()
                        draw_panel(left_win, left_items, left_idx, active_panel == 'left', left_offset, disk_mode=left_disk_mode)
                        draw_panel(right_win, right_items, right_idx, active_panel == 'right', right_offset, disk_mode=right_disk_mode)
                        #
                        draw_bottom_bar(stdscr)

                    stdscr.nodelay(False)  # Non-blocking input
                # --- "#"
                refresh_panels()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('4'):# --------------------------------- INSERT measurement
                logging.debug(f" key {chr(key)} pressed   ")
                if (active_panel == 'left' and len(left_items) == 0 ) or (active_panel == 'right' and len(right_items) == 0 ):
                    show_message_box(stdscr, f"To create DATABASE, use key 7")
                    pass
                elif (active_panel == 'left' and left_disk_mode) or (active_panel == 'right' and right_disk_mode):
                    show_message_box(stdscr, "FEATURE NOT IMPLEMENTED")
                    pass
                else:
                    name = None
                    dbase = None
                    if active_panel == 'left':
                        name = left_items[left_idx]
                        dbase = left_db_selected
                        #other_dbase = right_db_selected
                    else:
                        name = right_items[right_idx]
                        dbase = right_db_selected
                        #other_dbase = left_db_selected

                    aaa = ''.join(chr(random.randint(97, 122)) for _ in range(3))
                    res = maninf.insert_new_measurement(f"z_test_{aaa}", dbase, silent=True)
                    logging.debug(f"{res}")
                    #show_message_box(stdscr, f"{res}")
                refresh_panels()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('5'):# --------------------------------- COPY measurement
                logging.debug(f" key {chr(key)} pressed   ")
                name = None
                dbase = None
                if (active_panel == 'left' and len(left_items) == 0 ) or (active_panel == 'right' and len(right_items) == 0 ):
                    pass
                elif not left_disk_mode and not right_disk_mode:
                    # -----------------------------------  all influx
                    if active_panel == 'left':
                        name = left_items[left_idx]
                        dbase = left_db_selected
                        other_dbase = right_db_selected
                    else:
                        name = right_items[right_idx]
                        dbase = right_db_selected
                        other_dbase = left_db_selected
                    logging.debug(f"copy no disk {name} from {dbase} to {other_dbase} ")
                    on_copy(name, dbase, other_dbase, stdscr)  # *********************************     '5' COPY FROM DBASE TO DBASE ******
                    refresh_panels()
                else:# ------------------- one panel is disk
                    if active_panel == 'left' and right_disk_mode: # COPY TO DISK L- R ----------------------------------
                        name = left_items[left_idx]
                        dbase = left_db_selected
                        if name and dbase: # copy measurement
                            #filename = input_dialog(stdscr, f"Export measurement '{name}' from {dbase} to file (in mirror folder): ")
                            #filename = f"{name}.csv" # NO, JUST PATH
                            filepath = DISK_MIRROR_PATH #  #filepath = os.path.join(DISK_MIRROR_PATH, filename)
                            logging.debug(f"copy to disk left2right: {name} from {dbase} to {filepath} ")
                            show_message_box(stdscr, f"... export2disk  ...", wait_key=False)
                            res = maninf.export_measurement2(name, dbase, filepath=filepath) # csv export
                            refresh_panels()
                            #show_message_box(stdscr, res)
                        elif name and dbase is None: # BACKUP
                            filename = f"{name}.exported" # THIS IS EXPORT DIR!
                            filepath = os.path.join(DISK_MIRROR_PATH, filename)
                            if os.path.exists(filepath):
                                show_message_box(stdscr, f"NO BACKUP TO EXISTING BACKUP \n{filepath}")
                            else:
                                logging.debug(f"copy DB to disk: {name}  to {filepath} ")
                                show_message_box(stdscr, f"... backup portable  ...", wait_key=False)
                                maninf.backup_portable(filepath, name ) # ONE  DB BACKUP
                                refresh_panels()

                    elif  active_panel == 'right' and left_disk_mode: # COPY TO DISK R - L -----------------------------------------
                        name = right_items[right_idx]
                        dbase = right_db_selected
                        if name and dbase: # copy measurement
                            #filename = input_dialog(stdscr, f"Export measurement '{name}' from {dbase} to file (in mirror folder): ")
                            #filename = f"{name}.csv" # NO, JUST PATH
                            filepath = DISK_MIRROR_PATH # filepath = os.path.join(DISK_MIRROR_PATH, filename)
                            logging.debug(f"copy to disk right2left: {name} from {dbase} to {filepath} ")
                            show_message_box(stdscr, f"... export2disk  ...", wait_key=False)
                            res = maninf.export_measurement2(name, dbase, filepath=filepath) # csv export
                            refresh_panels()
                            #show_message_box(stdscr, res)
                        elif name and dbase is None: # BACKUP
                            filename = f"{name}.exported"  # THIS IS EXPORT DIR!
                            filepath = os.path.join(DISK_MIRROR_PATH, filename)
                            if os.path.exists(filepath):
                                show_message_box(stdscr, f"NO BACKUP TO EXISTING BACKUP {filepath}")
                            else:
                                logging.debug(f"copy DB to disk: {name}  to {filepath} ")
                                maninf.backup_portable(filepath, name ) # ONE  DB BACKUP

                    elif active_panel == "right" and right_disk_mode: # * * * * * * *  RESTORE TO  INFLUX
                        #show_message_box(stdscr, "FEATURE NOT IMPLEMENTED\n copying disk to DB right side")
                        name = right_items[right_idx]
                        if name.find(".exported") > 0:
                            basename = name.split(".exported")[0]
                            filepath = os.path.join(DISK_MIRROR_PATH, name)
                            oldname = maninf.restore_portable_get_oldname(filepath, database=basename)
                            confirm = confirm_dialog(stdscr, f"RESTORE DATABASE:\n {basename} \n Original name: \n {oldname}")
                            if confirm: maninf.restore_portable(filepath, database=basename)
                        else:
                            show_message_box(stdscr, "FEATURE NOT IMPLEMENTED\n copying disk to infl right side")
                        pass
                    elif active_panel == "left" and left_disk_mode:  # * * * * * * * * RESTORE TO  INFLUX
                        name = left_items[left_idx]
                        if name.find(".exported") > 0:
                            basename = name.split(".exported")[0]
                            filepath = os.path.join(DISK_MIRROR_PATH, name)
                            oldname = maninf.restore_portable_get_oldname(filepath, database=basename)
                            confirm = confirm_dialog(stdscr, f"RESTORE DATABASE:\n {basename} \n Original name: \n {oldname}")
                            if confirm: maninf.restore_portable(filepath, database=basename)
                        else:
                            show_message_box(stdscr, "FEATURE NOT IMPLEMENTED\n copying disk to infl left side")
                        pass
                    else:
                        show_message_box(stdscr, "FEATURE NOT IMPLEMENTED")
                        pass
                refresh_panels() # refresh after copy


            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('6'):# --------------------------------- MOVE measurement
                logging.debug(f" key {chr(key)} pressed   ")
                if (active_panel == 'left' and len(left_items) == 0 ) or (active_panel == 'right' and len(right_items) == 0 ):
                    pass
                elif (active_panel == 'left' and left_disk_mode) or (active_panel == 'right' and right_disk_mode):
                    show_message_box(stdscr, "FEATURE NOT IMPLEMENTED")
                    pass
                else:
                    name = None
                    dbase = None
                    if active_panel == 'left':
                        name = left_items[left_idx]
                        dbase = left_db_selected
                        other_dbase = right_db_selected
                    else:
                        name = right_items[right_idx]
                        dbase = right_db_selected
                        other_dbase = left_db_selected
                    if dbase is None or other_dbase is None:
                        logging.debug(f"I am in db view and F6 db=={name} this time. TO RENAME")
                        newname = input_dialog(stdscr, f"Rename {name} to : ")
                        if len(newname) > 0:
                            maninf.create_database(newname)
                            # now copy  all measurements in
                            res = maninf.show_measurements(name)
                            logging.debug(f"About to copy this to {newname}: {res}")
                            for i in res:
                                logging.debug(f" - copying {i} from {name} to {newname}")
                                on_copy(i, name, newname, stdscr, asking=False)
                                logging.debug(f" - not dropping {i} in {name}; ")
                            name = input_dialog(stdscr, f"You drop {name} database yourself ")
                        else:logging.debug("empty string for newname")
                    else: # COPY AND DELETE
                        logging.debug(f"I am in measurement view and F6 mea=={name}")
                        on_copy(name, dbase, other_dbase, stdscr, wordcopy="MOVE")  # ********************* '6' COPY PART .....
                        refresh_panels() # prepare space for next dialog
                        on_delete_measurement(name, dbase, stdscr, asking=False) # ************************* '6' DELETE PART
                refresh_panels()
                #on_move()
                #refresh_panels()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('^'):  # SHIFT-6 rename measurement or disk file
                logging.debug(f" key {chr(key)} pressed   ")
                if active_panel == 'left':
                    if left_disk_mode:
                        # Rename file in disk mirror folder
                        selected_item = left_items[left_idx]
                        if selected_item == '..':
                            show_message_box(stdscr, "Cannot rename parent directory entry")
                        else:
                            old_path = os.path.join(DISK_MIRROR_PATH, selected_item.rstrip('/'))
                            new_name = input_dialog(stdscr, f"Rename file '{selected_item}' to: ")
                            if new_name:
                                logging.debug(f"  rename left new=   {new_name} ")

                                new_name = rename_with_extension_preserved(selected_item, new_name)
                                new_path = os.path.join(DISK_MIRROR_PATH, new_name)
                                logging.debug(f"  rename left new pa=   {new_path} ")
                                try:
                                    os.rename(old_path, new_path)
                                    #show_message_box(stdscr, f"Renamed '{selected_item}' to '{new_name}'")
                                except Exception as e:
                                    show_message_box(stdscr, f"Rename failed: {e}")
                                refresh_panels()
                    else:# ---------------normal influxdb mode---------------
                        # Rename measurement in influxdb
                        name = left_items[left_idx]
                        dbase = left_db_selected
                        logging.debug(f" key {chr(key)} pressed;  infl rename L; {name} from {dbase}   ")
                        if name and dbase: # measurement from database
                            new_name = input_dialog(stdscr, f"Rename measurement '{name}' to: ")
                            if new_name and new_name != name:
                                logging.debug(f" key {chr(key)} pressed;  infl rename; old={name} db={dbase}   ")
                                logging.debug(f" key {chr(key)} pressed;  infl rename; new={new_name} db={dbase}   ")
                                # Copy measurement to new_name then delete old
                                #on_copy
                                on_copy(name, dbase, dbase, stdscr, silent=True, new_measurement=new_name)  # ****************    '^' rename in same DBASE ***
                        elif dbase is None:
                            new_name = input_dialog(stdscr, f"Rename DB '{name}' to: ")
                            if len(new_name) > 0:
                                logging.debug(f"renaming database {name} to {new_name}")
                                maninf.create_database(new_name) # same as '6'
                                res = maninf.show_measurements(name)
                                for i in res:
                                    logging.debug(f" - copying {i} from {name} to {new_name}")
                                    on_copy(i, name, new_name, stdscr, asking=False)
                                    logging.debug(f" - not dropping {i} in {name}; ")
                                name = input_dialog(stdscr, f"You drop {name} database yourself ")
                            else:logging.debug("empty string for newname")

                else:# ---------- right panel is active ------------------------------------------
                    if right_disk_mode:
                        selected_item = right_items[right_idx]
                        if selected_item == '..':
                            show_message_box(stdscr, "Cannot rename parent directory entry")
                        else:# ---------------------------  on disk rename ???
                            old_path = os.path.join(DISK_MIRROR_PATH, selected_item.rstrip('/'))
                            new_name = input_dialog(stdscr, f"Rename file '{selected_item}' to: ")
                            logging.debug(f"  rename right new=   {new_name} ")
                            if new_name:
                                new_name = rename_with_extension_preserved(selected_item, new_name)
                                new_path = os.path.join(DISK_MIRROR_PATH, new_name)
                                logging.debug(f"  rename right new pa=   {new_path} ")
                                try:
                                    os.rename(old_path, new_path)
                                    #show_message_box(stdscr, f"Renamed '{selected_item}' to '{new_name}'")
                                except Exception as e:
                                    show_message_box(stdscr, f"Rename failed: {e}")
                                #refresh_panels()
                    else:# ---- influxdb mode ---------------
                        name = right_items[right_idx]
                        dbase = right_db_selected
                        logging.debug(f" key {chr(key)} pressed;  infl rename R; {name} {dbase}   ")
                        if name and dbase:
                            new_name = input_dialog(stdscr, f"Rename measurement '{name}' to: ")
                            if new_name and new_name != name:
                                logging.debug(f" key {chr(key)} pressed;  infl rename; old={name} db={dbase}   ")
                                logging.debug(f" key {chr(key)} pressed;  infl rename; new={new_name} db={dbase}   ")
                                #on_copy
                                on_copy(name, dbase, dbase, stdscr, silent=True, new_measurement=new_name)  # ****************    '^' rename in same DBASE ****
                        elif dbase is None:
                            new_name = input_dialog(stdscr, f"Rename DB '{name}' to: ")
                            if len(new_name) > 0:
                                logging.debug(f"renaming database {name} to {new_name}")
                                maninf.create_database(new_name) # same as '6'
                                res = maninf.show_measurements(name)
                                for i in res:
                                    logging.debug(f" - copying {i} from {name} to {new_name}")
                                    on_copy(i, name, new_name, stdscr, asking=False)
                                    logging.debug(f" - not dropping {i} in {name}; ")
                                name = input_dialog(stdscr, f"You drop {name} database yourself ")
                            else:logging.debug("empty string for newname")

                                #res_copy = maninf.copy_measurement(name, dbase, dbase, silent=True, new_measurement_name=new_name)
                                #logging.debug(f" key {chr(key)} pressed;  infl rename; result: /{res_copy}/  ")
                                #if "error" not in res_copy.lower():
                                #    maninf.delete_measurement(name, dbase)
                                #    #show_message_box(stdscr, f"Renamed measurement '{name}' to '{new_name}'")
                                #else:
                                #    show_message_box(stdscr, f"Rename failed: {res_copy}")
                refresh_panels()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('7'):# --------------------------------- CREATE database
                logging.debug(f" key {chr(key)} pressed   ")
                if (active_panel == 'left' and left_disk_mode) or (active_panel == 'right' and right_disk_mode):
                    show_message_box(stdscr, "FEATURE NOT IMPLEMENTED")
                    pass
                else:
                    name = input_dialog(stdscr, "Create name: ")
                    on_create_database(name)
                refresh_panels()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('8'): # --------------------------------- DELETE database/measurement
                logging.debug(f" key {chr(key)} pressed   ")
                if (active_panel == 'left' and left_disk_mode) or (active_panel == 'right' and right_disk_mode):
                    show_message_box(stdscr, "FEATURE NOT IMPLEMENTED")
                    pass
                else:
                    logging.debug("8... in delete mode...")
                    if active_panel == 'left':
                        if left_items:
                            #logging.debug("8... deleting...left...")
                            #logging.debug(f"8... deleting...left...{left_idx}")
                            selected_item = left_items[left_idx]
                            logging.debug(f"8. deleting...selected: {selected_item}, db={left_db_selected}")
                            if (selected_item != '..') and (left_db_selected is None): #  view ??? meas?
                                logging.debug(f"*in delete DB and db_sel {selected_item}")
                                #name = left_items[left_idx]
                                on_delete_database(selected_item, stdscr)
                                #on_delete_measurement(name, left_db_selected, stdscr)
                            elif (left_db_selected is not None): #  view datab ???
                                name = left_items[left_idx]
                                # name== selected_item
                                logging.debug(f"*in delete MEAS and Me={name} sel=={selected_item} DB={left_db_selected}")
                                on_delete_measurement(selected_item, left_db_selected, stdscr)
                    if active_panel == 'right':
                        if right_items:# right panel  ***********************
                            logging.debug("8... deleting...right ... TO DO !!")
                            selected_item = right_items[right_idx]
                            logging.debug(f"8. deleting...selected: {selected_item}, db={right_db_selected}")
                            if (selected_item != '..') and (right_db_selected is None): #  view ??? meas?
                                logging.debug(f"*in delete DB and db_sel {selected_item}")
                                on_delete_database(selected_item, stdscr)
                            elif (right_db_selected is not None): #  view datab ???
                                name = right_items[right_idx]
                                logging.debug(f"*in delete MEAS and Me={name} sel=={selected_item} DB={right_db_selected}")
                                on_delete_measurement(selected_item, right_db_selected, stdscr)
                    # -----------------------------------------------------
                refresh_panels()


            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('9'):# --------------------------------- GRANT ALL PRIVILEGES (admin only)
                logging.debug(f" key {chr(key)} pressed   ")
                if not maninf.user_glob_admin:
                    show_message_box(stdscr, "ERROR: Must be in admin mode (-a) to grant privileges")
                elif (active_panel == 'left' and left_disk_mode) or (active_panel == 'right' and right_disk_mode):
                    show_message_box(stdscr, "FEATURE NOT AVAILABLE IN DISK MODE")
                elif (active_panel == 'left' and len(left_items) == 0) or (active_panel == 'right' and len(right_items) == 0):
                    show_message_box(stdscr, "No database selected")
                else:
                    # We need to be at database level (not measurement level)
                    db_name = None
                    if active_panel == 'left':
                        if left_db_selected is None:
                            # We're at database level
                            db_name = left_items[left_idx]
                        else:
                            show_message_box(stdscr, "Please select a DATABASE (go back with Enter on '..')")
                    else:
                        if right_db_selected is None:
                            # We're at database level
                            db_name = right_items[right_idx]
                        else:
                            show_message_box(stdscr, "Please select a DATABASE (go back with Enter on '..')")

                    if db_name is not None:
                        # Get the normal username
                        original_admin_state = maninf.user_glob_admin
                        maninf.user_glob_admin = False
                        normal_username, _, _ = maninf.get_user_pass()
                        maninf.user_glob_admin = original_admin_state

                        confirm = confirm_dialog(stdscr, f"Grant ALL PRIVILEGES on '{db_name}'\nto user '{normal_username}'?")
                        if confirm:
                            show_message_box(stdscr, "... granting privileges ...", wait_key=False)
                            maninf.grant_all_privileges(db_name, username=normal_username, silent=False)
                            show_message_box(stdscr, f"Granted ALL PRIVILEGES on '{db_name}' to '{normal_username}'")
                            logging.debug(f"Granted ALL PRIVILEGES on {db_name} to {normal_username}")
                        else:
                            logging.debug(f"Grant cancelled for {db_name}")
                refresh_panels()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('r'): # --------------------------------- refresh panels
                logging.debug(f" key {chr(key)} pressed   ")
                refresh_panels()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('d'):  # ----------------------------------- CHANGE  DISK MODE --------------
                logging.debug(f" key {chr(key)} pressed   ")
                if active_panel == 'left':
                    left_disk_mode = not left_disk_mode
                    if left_disk_mode:
                        # Reset selection and offset for disk mode
                        left_idx = 0
                        left_offset = 0
                    else:
                        # Reset selection and offset for influxdb mode
                        left_idx = 0
                        left_offset = 0
                else:
                    right_disk_mode = not right_disk_mode
                    if right_disk_mode:
                        right_idx = 0
                        right_offset = 0
                    else:
                        right_idx = 0
                        right_offset = 0
                refresh_panels()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key == ord('b'): # --------------------------------- backup ALL  ... to the hardcoded path
                logging.debug(f" key {chr(key)} pressed   ")
                on_backup(stdscr)
                refresh_panels()

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key in (curses.KEY_ENTER, 10, 13): # movements throug influxdb or view basics info on measurement
                logging.debug(f" key ENTER pressed   ")

                if active_panel == 'left':
                    #logging.debug(f"Left ENTER db-selected=={left_db_selected}")
                    # ----
                    if left_disk_mode: # NEW DISK HANDLING ************************** DISK *****
                        selected_item = left_items[left_idx]
                        path = os.path.join(DISK_MIRROR_PATH, selected_item.rstrip('/'))
                        if selected_item == '..':
                            # Go up one directory if inside subdir, else stay
                            parent = os.path.dirname(DISK_MIRROR_PATH)
                            # For simplicity, stay in root mirror folder (no going above)
                            pass
                        elif os.path.isdir(path):
                            # Optional: implement subdir navigation if desired
                            text = ""
                            if selected_item.find(".exported") > 0:
                                nm1 = selected_item.split(".exported")[0]
                                filepath = os.path.join(DISK_MIRROR_PATH, selected_item)
                                oldname = maninf.restore_portable_get_oldname(filepath, database=nm1)
                                text = f"... exported database: \n {nm1} \n ... prev name: \n {oldname}"

                            show_message_box(stdscr, f"Directory: \n {selected_item} \n {text}")
                        elif os.path.isfile(path): # When file, show content :)
                            try:
                                with open(path, 'r') as f:
                                    content = f.read(1024)
                                show_message_box(stdscr, f"File: {selected_item}\n\n{content}")
                            except Exception as e:
                                show_message_box(stdscr, f"Error reading file: {e}")
                        refresh_panels()
                    elif left_items:#   PREVIOUS HANDLING influxdb-----------------------------------
                        selected_item = left_items[left_idx]
                        #logging.debug(f"Left ENTER item-selected=={selected_item}")
                        if selected_item == '..': # going up
                            left_db_selected = None
                            left_items = get_left_panel_status()
                            left_idx = left_idx_prev # not 0
                        elif left_db_selected is None:
                            left_db_selected = selected_item
                            left_idx_prev = left_idx # store  previous index
                            left_items = ['..'] + maninf.show_measurements(left_db_selected)
                            left_idx = 0
                        else:
                            logging.debug(f"enter on measurement: see newest; m1={selected_item} db1={left_db_selected}")
                            res = maninf.show_measurement_newest_oldest(selected_item, left_db_selected, silent=True)

                            logging.debug(f"{res}")
                            show_message_box(stdscr, res)
                            pass
                else: # the active panel is right ---------

                    #logging.debug(f"Right ENTER db-selected=={right_db_selected}")
                    # ----
                    if right_disk_mode: # NEW DISK HANDLING ************************** DISK *****
                        selected_item = right_items[right_idx]
                        path = os.path.join(DISK_MIRROR_PATH, selected_item.rstrip('/'))
                        if selected_item == '..':
                            # Go up one directory if inside subdir, else stay
                            parent = os.path.dirname(DISK_MIRROR_PATH)
                            # For simplicity, stay in root mirror folder (no going above)
                            pass
                        elif os.path.isdir(path):
                            # Optional: implement subdir navigation if desired
                            text = ""
                            if selected_item.find(".exported") > 0:
                                nm1 = selected_item.split(".exported")[0]
                                filepath = os.path.join(DISK_MIRROR_PATH, selected_item)
                                oldname = maninf.restore_portable_get_oldname(filepath, database=nm1)
                                text = f"... exported database: \n {nm1} \n ... prev name: \n {oldname}"
                            show_message_box(stdscr, f"Directory: \n {selected_item} \n {text}")
                        elif os.path.isfile(path):
                            try:
                                with open(path, 'r') as f:
                                    content = f.read(1024)
                                show_message_box(stdscr, f"File: {selected_item}\n\n{content}")
                            except Exception as e:
                                show_message_box(stdscr, f"Error reading file: {e}")
                        refresh_panels()

                    elif right_items: #   PREVIOUS HANDLING influxdb-----------------------------------
                        selected_item = right_items[right_idx]
                        if selected_item == '..':
                            right_db_selected = None
                            right_items = get_right_panel_status()
                            right_idx = right_idx_prev # 0
                        elif right_db_selected is None:
                            right_db_selected = selected_item
                            right_idx_prev = right_idx # store
                            right_items = ['..'] + maninf.show_measurements(right_db_selected)
                            right_idx = 0
                        else:
                            logging.debug(f"enter on measurement: see newest; m1={selected_item} db1={right_db_selected}")
                            res = maninf.show_measurement_newest_oldest(selected_item, right_db_selected, silent=True)

                            logging.debug(f"{res}")
                            show_message_box(stdscr, res)
                            pass

            # elif key in (curses.KEY_ENTER, 10, 13):
            #     if active_panel == 'left':
            #         if left_items:
            #             left_db_selected = left_items[left_idx]
            #             left_items = maninf.show_measurements(left_db_selected)
            #             left_idx = 0
            #     else:
            #         if right_items:
            #             right_db_selected = right_items[right_idx]
            #             right_items = maninf.show_measurements(right_db_selected)
            #             right_idx = 0

            # ================================================================================
            #
            # --------------------------------------------------------------------------------

            elif key in (ord('q'), 27):  # q or ESC to quit
                break

            #  newer with scrolling
            draw_panel(left_win, left_items, left_idx, active_panel == 'left', left_offset, disk_mode=left_disk_mode)
            draw_panel(right_win, right_items, right_idx, active_panel == 'right', right_offset, disk_mode=right_disk_mode)
            #
            draw_bottom_bar(stdscr)

    curses.wrapper(curses_main)

if __name__ == "__main__":
    #   LOGLIFE DEFINED IN click MAIN....
    main()



"""Main CLI entry point for the disk manager application."""

import click
import curses
import sys
import time

# from p20260130_12_kilo01.disk_scanner import scan_disks
# from p20260130_12_kilo01.ui import DiskManagerUI
from jusfltuls.disk_scanner import scan_disks
from jusfltuls.shell_calls import (
    run_command,
    build_sudo_smartctl_x,
    build_sudo_btrfs_device_stats,
    build_sudo_btrfs_scrub_status,
    build_sudo_snapper_c_list,
    build_nvme_smart_log,
    build_nvme_device_self_test_status,
    set_debug_mode,
)
from jusfltuls.nvme_cli import check_nvme_cli_available, is_nvme_device
from jusfltuls.ui import DiskManagerUI


def check_nvme_tools(disks: list) -> tuple[bool, str]:
    """Check if nvme utility is installed when NVMe devices are present.

    Args:
        disks: List of Disk objects from scan_disks()

    Returns:
        Tuple of (ok, message) - ok is True if no NVMe devices or nvme is available
    """
    has_nvme = any(is_nvme_device(disk.device_path) for disk in disks)

    if not has_nvme:
        return True, ""

    # Check if nvme CLI is available
    if not check_nvme_cli_available():
        return False, (
            "NVMe devices detected but 'nvme' CLI utility is not installed.\n"
            "Please install nvme-cli package:\n"
            "  - Debian/Ubuntu: sudo apt install nvme-cli\n"
            "  - RHEL/CentOS: sudo yum install nvme-cli\n"
            "  - Arch: sudo pacman -S nvme-cli"
        )

    return True, ""


def check_sudo_access() -> tuple[bool, list[str]]:
    """Check if all required sudo commands can run without password.

    Returns:
        Tuple of (all_ok, list_of_failed_commands)
    """
    # List of commands to test with a sample device/mountpoint
    # We prepend "-n" to tell sudo not to prompt for password
    test_commands = [
        ("smartctl", lambda: ["sudo", "-n"] + build_sudo_smartctl_x("/dev/null")[1:]),
        (
            "btrfs device stats",
            lambda: ["sudo", "-n"] + build_sudo_btrfs_device_stats("/dev/null")[1:],
        ),
        (
            "btrfs scrub status",
            lambda: ["sudo", "-n"] + build_sudo_btrfs_scrub_status("/")[1:],
        ),
        (
            "snapper list",
            lambda: ["sudo", "-n"] + build_sudo_snapper_c_list("root")[1:],
        ),
        ("nvme smart-log", lambda: ["sudo", "-n"] + build_nvme_smart_log("/dev/null")[1:]),
        ("nvme device-self-test", lambda: ["sudo", "-n"] + build_nvme_device_self_test_status("/dev/null")[1:]),
    ]

    failed_commands = []

    for name, cmd_builder in test_commands:
        try:
            cmd = cmd_builder()
            returncode, output = run_command(cmd, timeout=5)

            # Check if sudo requires password (sudo -n returns error if password needed)
            # Typical error: "sudo: a password is required" or exit code != 0
            if returncode != 0:
                # Check for common password-related error messages
                output_lower = output.lower()
                if any(
                    msg in output_lower
                    for msg in ["password is required", "password:", "sorry, try again"]
                ):
                    failed_commands.append(name)
                elif "sudo" in output_lower and returncode == 1:
                    # Likely permission denied or requires password
                    failed_commands.append(name)
        except Exception:
            failed_commands.append(name)

    return len(failed_commands) == 0, failed_commands


def show_sudo_warning(failed_commands: list[str]) -> bool:
    """Show warning about missing sudo permissions and ask user what to do.

    Returns:
        True if user wants to continue, False to quit
    """
    print("\n" + "=" * 70, file=sys.stderr)
    print("  SUDO PERMISSION CHECK FAILED", file=sys.stderr)
    print("=" * 70, file=sys.stderr)
    print("\nThe following commands require passwordless sudo access:", file=sys.stderr)
    print("\n  Commands that need visudo configuration:", file=sys.stderr)
    for cmd in failed_commands:
        print(f"    - {cmd}", file=sys.stderr)

    print("\n" + "-" * 70, file=sys.stderr)
    print("REQUIRED VISUDO CONFIGURATION:", file=sys.stderr)
    print("-" * 70, file=sys.stderr)
    print("\nAdd this line to /etc/sudoers (using 'sudo visudo'):", file=sys.stderr)
    print(
        "\n  myuser ALL=(ALL) NOPASSWD: /usr/sbin/smartctl, /usr/bin/btrfs, /usr/bin/snapper, /usr/sbin/nvme",
        file=sys.stderr,
    )
    print("\nReplace 'myuser' with your actual username.", file=sys.stderr)
    print(
        "\nOr run this command (replace USERNAME with your username):", file=sys.stderr
    )
    print(
        '  echo "USERNAME ALL=(ALL) NOPASSWD: /usr/sbin/smartctl, /usr/bin/btrfs, /usr/bin/snapper, /usr/sbin/nvme" | sudo tee /etc/sudoers.d/diskmanager',
        file=sys.stderr,
    )
    print("-" * 70, file=sys.stderr)
    print("\nWithout passwordless sudo, some features will not work.", file=sys.stderr)
    print("=" * 70 + "\n", file=sys.stderr)

    while True:
        response = input("Continue anyway? [y/N]: ").strip().lower()
        if response in ("y", "yes"):
            return True
        elif response in ("n", "no", ""):
            return False
        else:
            print("Please enter 'y' or 'n'")


def run_ui(stdscr, refresh_interval: int = 10) -> None:
    """Main UI loop using curses.

    Args:
        stdscr: Curses window object
        refresh_interval: Auto-refresh interval in seconds
    """
    # Initialize UI
    ui = DiskManagerUI(stdscr)
    ui.left_panel.is_active = True

    # Initial disk scan
    disks, error = scan_disks()
    if error:
        ui.set_error(error)
    ui.update_disks(disks)

    # Load health info for all disks on startup
    for idx in range(len(disks)):
        ui.load_disk_health(idx)
    ui._update_left_panel()

    # Draw initial UI
    ui.draw()

    # Track last refresh time
    last_refresh = time.time()

    # Main loop - only redraw on user input or timer
    stdscr.nodelay(True)  # Non-blocking input
    while True:
        # Get user input (non-blocking)
        try:
            key = stdscr.getch()
        except KeyboardInterrupt:
            break

        # Check for auto-refresh
        current_time = time.time()
        needs_redraw = False

        if refresh_interval > 0 and (current_time - last_refresh) >= refresh_interval:
            # Auto-refresh: reload health info for all disks and update scrub status
            ui.set_status("refresh")
            for idx in range(len(ui.disks)):
                ui.load_disk_health(idx)
            ui._update_left_panel()
            ui.update_scrub_status()
            ui._update_all_partitions_panel()
            last_refresh = current_time
            needs_redraw = True

        # Also update scrub status on every keypress for responsiveness
        if key != -1:
            ui.update_scrub_status()

        # Handle input
        if key == ord("q") or key == ord("Q"):
            break
        elif key == ord("r") or key == ord("R"):
            # Manual refresh - reload health info for all disks to update test progress
            disks, error = scan_disks()
            if error:
                ui.set_error(error)
            else:
                ui.clear_messages()
            ui.update_disks(disks)
            # Reload health info for all disks to get updated test progress
            for idx in range(len(disks)):
                ui.load_disk_health(idx)
            ui._update_left_panel()
            needs_redraw = True
        elif key == ord("t") or key == ord("T"):
            # Start short self-test for selected disk
            if ui.active_panel == "left":
                selected_idx = ui.left_panel.selected_index
                success, message = ui.start_short_test(selected_idx)
                if success:
                    ui.set_status(message)
                else:
                    ui.set_error(message)
                ui._update_left_panel()
                needs_redraw = True
        elif key == ord("l") or key == ord("L"):
            # Start long self-test for selected disk
            if ui.active_panel == "left":
                selected_idx = ui.left_panel.selected_index
                success, message = ui.start_long_test(selected_idx)
                if success:
                    ui.set_status(message)
                else:
                    ui.set_error(message)
                ui._update_left_panel()
                needs_redraw = True
        elif key == ord("s") or key == ord("S"):
            # Start btrfs scrub for selected partition in right panel
            if ui.active_panel == "right":
                selected_idx = ui.right_panel.selected_index
                success, message = ui.start_btrfs_scrub(selected_idx)
                if success:
                    ui.set_status(message)
                    ui._update_all_partitions_panel()
                else:
                    ui.set_error(message)
                needs_redraw = True
        elif key == ord("\t") or key == 9:  # Tab key
            ui.switch_panel()
            needs_redraw = True
        elif key == curses.KEY_UP:
            ui.move_up()
            needs_redraw = True
        elif key == curses.KEY_DOWN:
            ui.move_down()
            needs_redraw = True
        elif key == curses.KEY_LEFT:
            if ui.active_panel == "right":
                ui.switch_panel()
                needs_redraw = True
        elif key == curses.KEY_RIGHT:
            if ui.active_panel == "left":
                ui.switch_panel()
                needs_redraw = True

        # Redraw when something changed
        if needs_redraw:
            ui.draw()

        # Small sleep to prevent CPU spinning
        time.sleep(0.1)


@click.command()
@click.option(
    "--refresh",
    "-r",
    default=10,
    help="Auto-refresh interval in seconds (0 to disable).",
    type=int,
)
@click.option(
    "--debug",
    "-d",
    is_flag=True,
    default=False,
    help="Enable debug mode to show all command executions.",
)
@click.version_option(package_name="jusfltuls", prog_name="jusfltuls")
def main(refresh: int, debug: bool) -> None:
    """
    Midnight Commander-style disk manager.

    Displays disks and partitions in a two-panel interface.
    Navigate with arrow keys, switch panels with Tab, quit with 'q'.
    """
    # Set global debug mode
    set_debug_mode(debug)

    # Check sudo access before starting
    all_ok, failed_commands = check_sudo_access()
    if not all_ok:
        if not show_sudo_warning(failed_commands):
            print(
                "\nExiting. Please configure sudo permissions and try again.",
                file=sys.stderr,
            )
            sys.exit(1)
        print("\nContinuing without full sudo permissions...\n", file=sys.stderr)

    # Scan disks and check for nvme tools if needed
    disks, error = scan_disks()
    if error:
        print(f"\nError scanning disks: {error}", file=sys.stderr)
        sys.exit(1)

    nvme_ok, nvme_message = check_nvme_tools(disks)
    if not nvme_ok:
        print("\n" + "=" * 70, file=sys.stderr)
        print("  NVME TOOLS NOT FOUND", file=sys.stderr)
        print("=" * 70, file=sys.stderr)
        print(f"\n{nvme_message}", file=sys.stderr)
        print("=" * 70 + "\n", file=sys.stderr)

    try:
        # Run the curses UI with refresh interval
        curses.wrapper(lambda stdscr: run_ui(stdscr, refresh))
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


if __name__ == "__main__":
    main()




#     when problem in h5
# pip3 install --upgrade tables
# pip3 install hd5
# apt  install  lbzip2
# apt  install
# apt  install p7zip-full
#
import pandas as pd
import subprocess as sp
import os
#import click
import datetime as dt
import numpy as np
import matplotlib.pyplot as plt
#import h5py

import sys
import subprocess
from console import fg, bg, fx
#from fire import Fire


from jusfltuls.check_new_version import is_there_new_version

comps=['pixz','pbzip2','lbzip2','pigz','zstd','7z']
# pbzip is bad with image... very long time
comps=['pixz','lbzip2','pigz','zstd','7z']






FILESIZE=-1

def file_size(file_path):
    """
    this function will return the file size
    """
    global FILESIZE
    #if FILESIZE>=0:
    #    return FILESIZE
    if os.path.isfile(file_path):
        #print("D... finding file size")
        #print("D... isfile")
        file_info = os.stat(file_path)
        #print( file_info.st_size)
        FILESIZE=file_info.st_size
    else:
        #print("D... I assume it is a FOLDER")
        out=subprocess.check_output(['du','-bs', file_path ])
        #print(out)
        FILESIZE= out.split()[0].decode('utf-8')
        FILESIZE=int(FILESIZE)
    #print("D.... Size=",FILESIZE)
    #quit()
    return FILESIZE






def get_max_level(code):
    if code=="xz" or code=="pixz":
        return 9
    elif code=="pbzip2" or code=="bzip2":
        return 9
    elif code=="lbzip2":
        return 9
    elif code=="pigz" or code=="gzip":
        return 9
    elif code=="zstd":
        return 19
    elif code=="7z":
        return 9
    else:
        print("!... not known compressor")
        quit()

def get_ext(code):
    if code=="xz" or code=="pixz":
        return 'xz'
#    elif code=="pbzip2" or code=="bzip2":
#        return 'bz2'
    elif code=="lbzip2":
        return 'bz2'
    elif code=="pigz" or code=="gzip":
        return 'gz'
    elif code=="zstd":
        return 'zst'
    elif code=="7z":
        return '7z'
    else:
        print("!... not known compressor")
        quit()


#@click.command()
#@click.argument('filename')
#@click.option('--code', '-c')
#@click.option('--level', '-l' ,default=1 )
def compfile( filename, code , level=1 , show_only=False):
    """
    create commandline for a specific tool
    """
    if filename[-1]=="/":
            filename = filename[:-1]

    size=file_size( filename )
    ext=get_ext(code)
    OUTFILE="{}.{}.tar.{}".format(  filename,level,ext )
    OUTFILE=os.path.split(OUTFILE)[-1]

    if code=="xz" or code=="pixz":
#        code="pxz --stdout -"+str(level)+" > "+OUTFILE
        code="pixz -"+str(level)+" > "+OUTFILE

    elif code=="pbzip2" or code=="bzip2":
        code="pbzip2 --stdout -"+str(level)+" > "+OUTFILE

    elif code=="lbzip2":
        code="lbzip2 --stdout -"+str(level)+" > "+OUTFILE

    elif code=="pigz" or code=="gzip":
        code="pigz --stdout -"+str(level)+" > "+OUTFILE

    elif code=="zstd":
        code="zstd -T0 --stdout -"+str(level)+" > "+OUTFILE

    elif code=="7z":
        code="7z a -si  -mx"+str(level)+" "+OUTFILE

    else:
        print("!... not known compressor")
        sys.exit(1)



    #CMD="cat {} | pv -s {} -r -e -p -t | {}  ".format(
    CMD="tar -c {} | pv -s {} -r -e -p -t | {}  ".format(
        filename ,
        size ,
        code)

    if show_only:
        print(CMD)
        return
    print(CMD)


    start=dt.datetime.now()
    ok = False
    try:
        sp.getoutput( CMD )
        ok = True
    except:
        ok = False
    stop=dt.datetime.now()


    X= (stop-start).total_seconds()
    if not ok:
        print("X... failed ", code.split()[0], float(level))
        return code.split()[0], float(level), float(X), float(0.0)


    print(OUTFILE, filename)
    print("D... ratio = {} /  {} ".format( file_size( OUTFILE ) , file_size( filename ) ) )
    Y=100*file_size( OUTFILE )/file_size( filename )
    #print( X)
    print( "{:10s}  {:2d}    {:6.1f}  {:3.1f} %".format( code.split()[0], level, X, Y ) )
    #  from 2021 -  level must be float not int
    return code.split()[0], float(level), float(X), float(Y)
#=compfile end=




#---------------------
def suggest(FILE):

    for co in comps:
        for i in range(1,get_max_level(co)+1 ):
            compfile( FILE  , co, i   , show_only=True)
    print("D.... add 't' as test to the command line  to run all compressions!!!")

    print("i... max comp choice and medium comp with fast decompression:")
    compfile( FILE  , "pixz", 9   , show_only=True)
    compfile( FILE  , "zstd", 10   , show_only=True)


#---------------------
def compress(FILE ):
    """
    MAKE ALL
    """
    df = pd.DataFrame( [], columns = ['time','ratio','code','level'] )

    ax = plt.gca()

    row=0
    for co in comps: # all compressors
        for i in range(1,get_max_level(co)+1 ): # all levels
        #for i in range(1,3 ): # all levels

            print(fg.green, f"=========== {co:6s} {i:3d}          ==========", fg.default)
            # *****************  RUN REAL
            code,level,x,y=compfile( FILE  , co, i, show_only=False)
            # ***************
            print(f"----------- {code:6s} {level:4.0f}  Time:{x:6.1f}   Ratio:{y:5.1f} % ----------" )
            print(".")


            df.loc[row]=[x,y,code,level]
            row=row+1
            #print("D...",df)
        pie=df[df.code==co].plot( style=".-",x='time',y='ratio', label=co,ax=ax)



    filename='compressed_results.h5'
    key="comp_"+FILE
    key = key.replace(".","_")
    key = key.replace("/","_")
    #    df.to_hdf(filename, key , table=True, mode='a')
    #print("\n\nD... {} {} \n\n".format( filename, key) )

    #    df.to_hdf(filename, key , format='table', mode='a')
    #print(df.dtypes)
    print(f"Filename: {filename}    Key:{key}")
    df.to_hdf(filename, key=key , format='table', mode='a')

    fig = pie.get_figure()
    fig.savefig("comp.png")
    #    plt.show()
    #    plt.savefig("comp.png")


    # print("D... VISIT LINE ======================+++")
    # with h5py.File(filename,'r') as hf:
    #     hf.visit(print)


    f=pd.read_hdf(filename,key)
    print("D... ========================= rpint hdf==========")
    print(f)



def decompress(FILE):
    print("D... decompress placeholder", FILE)
    ex = os.path.splitext(FILE)[-1]
    #print(ex)
    CMD = "echo Nocommand"
    if ex == ".zst":
        CMD = "zstd -d --stdout "+FILE+" > /dev/null"
    elif ex == ".7z":
        CMD = "7z x -so  "+FILE+" > /dev/null"
    elif ex == ".gz":
        CMD = "gunzip -d -c  "+FILE+" > /dev/null"
    elif ex == ".xz":
        CMD = "xz --decompress --stdout  "+FILE+" > /dev/null"
    elif ex == ".bz2":
        CMD = "lbzip2 --decompress --stdout  "+FILE+" > /dev/null"

    print(CMD)
    start=dt.datetime.now()
    sp.getoutput( CMD )
    stop=dt.datetime.now()

    X= (stop-start).total_seconds()
    print("{}  {:5.0f} sec.  {}".format(ex, X, FILE))


def main():


    is_there_new_version(package="jusfltuls", printit=True, printall=True)

    print(sys.argv)
    if len(sys.argv) < 2:
        print("X... use argument:    c or d or t  ... and File/Dir")
    elif len(sys.argv) < 3:
        print("X... use argument c or d or t and File/Dir ***")
        print("X... use argument c or d or t and File/Dir ***")
        print("X... use argument c or d or t and File/Dir ***")
    elif sys.argv[1] == "c":
        if os.path.exists( os.path.expanduser(sys.argv[2])):
            suggest(sys.argv[2])
        else:
            print("X...  BAD FILE/PATH")

    elif sys.argv[1] == "t":
        if os.path.exists( os.path.expanduser(sys.argv[2])):
            compress(sys.argv[2])
        else:
            print("X...  BAD FILE/PATH")

    elif sys.argv[1] == "d":
        if os.path.exists( os.path.expanduser(sys.argv[2])):
            decompress(sys.argv[2])
        else:
            print("X...  BAD FILE/PATH")
    else:
        print("X... some wrong argument")

######################################################
#
#
#
#######################################################
#
#
#
####################################################

if (__name__ == "__main__" ):
    help="""
i... BEST CHOICES:
 9.xz    Best compression (and faster than 7z) -
10.zst   Medium compression - lightning decompression
pixz     max and fast for RPi images

    """
    print(help)
    main()
    print(help)
